
const St = imports.gi.St;
const Main = imports.ui.main;
const Lang = imports.lang;
const Shell = imports.gi.Shell;
const Mainloop = imports.mainloop;
const AppDisplay = imports.ui.appDisplay;
const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Windows = Extension.imports.windows;
const Clutter = imports.gi.Clutter;

const AppFavorites = imports.ui.appFavorites;

const THUMBNAIL_SIZE = 300;

let taskbar;
let tasksList = new Array();
let windows = null;
let desktopView = null;
let previews = new Array();
let favorites = new Array();

let signalPreview = null;

function createTaskbar() {
	taskbar = new St.BoxLayout();
	
	taskbar.add_actor(buildDesktopBox());
	
	addFavorites();
	
	windows = new Windows.Windows(this, onWindowsListChange, onWindowChange);
}

function onWindowsListChange(windowsList, type, window) {
	desktopView = false;
	switch(type) {
		case 0: //init or workspace change
			refreshTasksList(windowsList);
			break;
			
		case 1: //add window
			addTaskInList(window);
			break;
			
		case 2: //remove window
			removeTaskInList(window);
			break;
	}
}

function onWindowChange(window, type) {
	if (type == 0) //Focus
	{
		this.tasksList.forEach(
			function(task)
			{
				let [windowTask, taskButton, signalsTask] = task;
				
				if(taskButton!=null) {
					if (windowTask == window)
						taskButton.add_style_pseudo_class('active-task');
					else
						taskButton.remove_style_pseudo_class('active-task');
				}
			}
		);
	}

	desktopView = false;
}

function buildDesktopBox() {
	let desktopIcon = new St.Icon({
		icon_name : "Show Desktop-symbolic", icon_size : 22
	});
	
	let desktopButton = new St.Button({ style_class: "panel-button" });
	desktopButton.connect("clicked", Lang.bind(this, onClickDesktopButton));
	desktopButton.set_child(desktopIcon);
	
	let desktopBox = new St.BoxLayout({ style_class: "tkb-desktop-box" });
	desktopBox.add_actor(desktopButton);
	
	return desktopBox;
}

function refreshTasksList(windowsList) {
	clearTasksList();
	
	windowsList.forEach(
		function(window)
		{
		    addTaskInList(window);
		}
    );
}

function addFavorites() {
	let launchers = global.settings.get_strv(AppFavorites.getAppFavorites().FAVORITE_APPS_KEY);
	
	for ( let i=0; i<launchers.length; ++i ) {
        let app = Shell.AppSystem.get_default().lookup_app(launchers[i]);
        
        if ( app == null ) {
            continue;
        }
        
        print("favorite name="+app.get_name());
        addFavorite(app,i);
	}
}

function addFavorite(app,i) {
	let taskIcon = new AppDisplay.AppIcon(app, { setSizeManually: true, showLabel: false });
	taskIcon.setIconSize(22);

	let taskButton = new St.Button({ style_class: "tkb-task-button" });
	taskButton.set_child(taskIcon.actor);
														
	let signal = taskButton.connect("button-press-event", Lang.bind(this,function() {
		app.open_new_window(-1);
	}));
	
	taskbar.insert_child_at_index(taskButton,i+1);
	
    favorites.push([app,taskButton,i,signal]);
}

function refreshFavorites(action, appName) {
	for(let i=0;i<favorites.length;i++){
		let favorite = favorites[i];
		
		if(favorite[0].get_name() == appName) {
			if(action == 'add') {
				favorite[1].disconnect(favorite[3]);
				favorite[1].destroy();
			}
			else if(action == 'remove'){
				addFavorite(favorite[0],favorite[2]);
				favorites.splice(i,1);
				break;
			}
		}
	}
}

function clearTasksList() {
	for (let i = tasksList.length - 1; i>=0; i--)
	{
		let [windowTask, buttonTask, signalsTask] = tasksList[i];
		signalsTask.forEach(
			function(signal)
			{
				buttonTask.disconnect(signal);
			}
		);
	
		buttonTask.destroy();
		tasksList.splice(i, 1);
	}
}

function addTaskInList(window) {
	let app = Shell.WindowTracker.get_default().get_window_app(window);
	
	let taskButton = null;
	let signalsTask = null;
	tasksList.forEach(
		function(task) {
			if(task[3] == app.get_name()){
				taskButton = task[1];
				signalsTask = task[2];
			}
		}
	);
		
	if(taskButton==null) {
		let taskIcon = new AppDisplay.AppIcon(app, { setSizeManually: true, showLabel: false });
		taskIcon.setIconSize(22);
	
		taskButton = new St.Button({ style_class: "tkb-task-button" });
		taskButton.set_child(taskIcon.actor);
		
		signalsTask = [
			taskButton.connect("button-press-event", Lang.bind(this, onClickTaskButton, window)),
			taskButton.connect("enter-event", Lang.bind(this, showPreviews, window)),
			taskButton.connect("leave-event", Lang.bind(this, hidePreviews))
		];
		
		if (window.has_focus())
			taskButton.add_style_pseudo_class('active-task');
		
		let position = null;
		favorites.forEach(
			function(favorite) {
				if(favorite[0].get_name() == app.get_name())
					position = favorite[2];
			}
		);
		
		if(position!=null)
			taskbar.insert_child_at_index(taskButton,position+1);
		else
			taskbar.add_actor(taskButton);
		
		refreshFavorites('add',app.get_name());
	}
	
	tasksList.push([ window, taskButton, signalsTask, app.get_name() ]);
}

function removeTaskInList(window)
{
	let index = findTaskIndex(window);
	
	let appName = tasksList[index][3];
	
	let count = 0;
	tasksList.forEach(
		function(task) {
			if(task[3] == appName){
				count++;
			}
		}
	);
	
	if (index != null && count == 1)
	{
		let [windowTask, buttonTask, signalsTask, appName] = this.tasksList[index];
		
		signalsTask.forEach(
			function(signal)
			{
				buttonTask.disconnect(signal);
			},
			this
		);
		
		refreshFavorites('remove',appName);
		
		buttonTask.destroy();
		destroyPreviews();
		tasksList.splice(index, 1);
		
		return true;
	}
	else
		return false;
}

function findTaskIndex(window) {
	for (let indexTask in this.tasksList)
	{
		let [windowTask, buttonTask, signalsTask] = this.tasksList[indexTask];
		
		if (windowTask == window)
			return indexTask;
	}
	
	return null;
}

function onClickDesktopButton(button, pspec)
{
	tasksList.forEach(
	    function(task)
	    {
	    	let [windowTask, buttonTask, signalsTask] = task;
	
	    	if (this.desktopView)
	    		windowTask.unminimize(global.get_current_time());
	    	else
	    	{
	    		windowTask.minimize(global.get_current_time());
	    		buttonTask.remove_style_pseudo_class('active-task');
	    	}
	    },
	    this
	)
}

function onClickTaskButton(button, pspec, window)
{
	//numButton is clicked button (left,middle,right)
	let numButton = pspec.get_button();
	
	if(numButton==1) {
		tasksList.forEach(
			function(task)
			{
				let [windowTask, buttonTask, signalsTask] = task;
		
				if (windowTask == window)
				{
					if (! windowTask.has_focus())
					{
						windowTask.activate(global.get_current_time());
						buttonTask.add_style_pseudo_class('active-task');
					}
					else
					{
						windowTask.minimize(global.get_current_time());
						buttonTask.remove_style_pseudo_class('active-task');
					}
				}
				else
					buttonTask.remove_style_pseudo_class('active-task');
			}
		);
		
		desktopView = false;
	}
}

function destroyPreviews() {
	for(let i=0;i<previews.length;i++) {
		let preview = previews[i];
		preview.disconnect(signalPreview);
		preview.destroy();
	}
	
	previews = new Array();
}

function showPreviews(button, pspec, window)
{
	//Remove current preview if necessary
	destroyPreviews();
	
	let app = Shell.WindowTracker.get_default().get_window_app(window);
	
	let i = 0;
	tasksList.forEach(
		function(task) {
			if(task[3] == app.get_name()) {
				let wnd = task[0];
				
				let preview = new St.Button({ style_class: "tkb-preview"});
				signalPreview = preview.connect("button-press-event", Lang.bind(this, onClickTaskButton, wnd));
				
				let thumbnail = getThumbnail(wnd, THUMBNAIL_SIZE);
				preview.add_actor(thumbnail);
							     
				Main.layoutManager.addChrome(preview);
				
				let [left, top] = button.get_transformed_position();
				
				let [width, height] = thumbnail.get_size();
				
				preview.set_position(left + (width * i) - 10, 22);
				
				previews.push(preview);
				
				i++;
			}
		}
	);
}

function getThumbnail(window, size)
{
	let thumbnail = null;
	let mutterWindow = window.get_compositor_private();
	
	if (mutterWindow)
	{
		let windowTexture = mutterWindow.get_texture();
		let [width, height] = windowTexture.get_size();
		let scale = Math.min(1.0, size / width, size / height);
		
		thumbnail = new Clutter.Clone ({ source: windowTexture, reactive: true, width: width * scale, height: height * scale });
	}
	
	return thumbnail;
}

function hidePreviews()
{
	//Destroy Preview if displaying and mouse not over it
	if (previews.length>0)
	{
		let [x, y, mods] = global.get_pointer();
		
		let [left, top] = previews[0].get_position();
		
		if((x<left || x>left+(THUMBNAIL_SIZE*previews.length) || y<top || y>top+THUMBNAIL_SIZE) && y > 24) {
			destroyPreviews();
		} else {
			if(previews.length>1) {
				for(let i=0;i<previews.length;i++) {
					if(x<left+(THUMBNAIL_SIZE*i))
						previews[i].add_style_pseudo_class('active');
					else
						previews[i].remove_style_pseudo_class('active');
				}
			} else {
				previews[0].add_style_pseudo_class('active');
			}
		}
		
		Mainloop.timeout_add(50, Lang.bind(this,hidePreviews));
	}
}

function init() {
	createTaskbar();
}

function enable() {
	Main.panel._leftBox.insert_child_at_index(taskbar, 1);
}

function disable() {
    Main.panel._leftBox.remove_child(taskbar);
}
