const Lang = imports.lang;
const Shell = imports.gi.Shell;

function Windows(callBackThis, callbackWindowsListChange, callbackWindowChange)
{
    this.init(callBackThis, callbackWindowsListChange, callbackWindowChange);
}

Windows.prototype =
{
    workspace: null,
    windowsList: new Array(),

    callBackThis: null,
    callbackWindowsListChange: null,
    callbackWindowChange: null,

    workspaceSwitchSignal: null,
    windowAddSignal: null,
    windowRemoveSignal: null,
    windowsSignals: new Array(),

    init: function(callBackThis, callbackWindowsListChange, callbackWindowChange)
    {
		//Set User Callback
		this.callBackThis = callBackThis;
		this.callbackWindowsListChange = callbackWindowsListChange;
		this.callbackWindowChanged = callbackWindowChange;
	
		//Init WindowsList
		this.onWorkspaceChange();
	
		//Add window manager signals
		this.workspaceSwitchSignal = global.window_manager.connect('switch-workspace', Lang.bind(this, this.onWorkspaceChange));
    },

    destruct: function()
    {
		//Remove window manager signals
		global.window_manager.disconnect(this.workspaceSwitchSignal);
	
		//Remove workspace signals
		this.workspace.disconnect(this.windowAddSignal);
		this.workspace.disconnect(this.windowRemoveSignal);
	
		//Clean windows list
		this.cleanWindowsList();
    },

    onWorkspaceChange: function()
    {
		//Remove workspace signals if necessary
		if (this.windowAddSignal != null)
		    this.workspace.disconnect(this.windowAddSignal);
		
		if (this.windowRemoveSignal != null)
		    this.workspace.disconnect(this.windowRemoveSignal);
	
		//Clean windows list
		this.cleanWindowsList();
	
		//Get current workspace
		this.workspace = global.screen.get_active_workspace();
	
		//Build windows list
		this.workspace.list_windows().forEach(
		    function(window)
		    {
		    	this.addWindowInList(window);
		    },
		    this
		);

		//Call User Callback
		this.callbackWindowsListChange.call(this.callBackThis, this.windowsList, 0, null);
	
		//Add workspace signals
		this.windowAddSignal = this.workspace.connect_after('window-added', Lang.bind(this, this.onWindowAdd));
		this.windowRemoveSignal = this.workspace.connect_after('window-removed', Lang.bind(this, this.onWindowRemove));
    },

    onWindowChanged: function(window, object, type)
    {
		if (type == 0) //Focus changed
		{
		    if (window.appears_focused)
			this.callbackWindowChanged.call(this.callBackThis, window, 0);
		}
		else if (type == 1) //Title changed
		    this.callbackWindowChanged.call(this.callBackThis, window, 1);
    },

    onWindowAdd: function(workspace, window)
    {
    	if (this.addWindowInList(window))
		    this.callbackWindowsListChange.call(this.callBackThis, this.windowsList, 1, window);
    },

    onWindowRemove: function(workspace, window)
    {
    	if (this.removeWindowInList(window))
		    this.callbackWindowsListChange.call(this.callBackThis, this.windowsList, 2, window);
    },

    searchWindowInList: function(window)
    {
		let index = null;
	
		for (let indexWindow in this.windowsList)
		{
		    if (this.windowsList[indexWindow] == window)
		    {
				index = indexWindow;
				break;
		    }
		}
	
		return index;
    },
    
    addWindowInList: function(window)
    {
		let index = this.searchWindowInList(window);
	
		if (index == null && ! window.is_skip_taskbar())
		{
		    this.windowsList.push(window);
	
		    //Add window signals
		    let objectAndSignals = [
				window,
				[
				    window.connect('notify::appears-focused', Lang.bind(this, this.onWindowChanged, 0)),
				    window.connect('notify::title', Lang.bind(this, this.onWindowChanged, 1))
				]
		    ];
		    this.windowsSignals.push(objectAndSignals);
	
		    return true;
		}
		else
		    return false;
    },

    removeWindowInList: function(window)
    {
		let index = this.searchWindowInList(window);
		
		if (index != null)
		{
		    this.windowsList.splice(index, 1);
	
		    //Remove window signals
		    for (let indexSignal in this.windowsSignals)
		    {
				let [object, signals] = this.windowsSignals[indexSignal];
		
				if (object == window)
				{
				    signals.forEach(
						function(signal)
						{
						    object.disconnect(signal);
						},
						this
				    );
		
				    this.windowsSignals.splice(indexSignal, 1);
		
				    break;
				}
		    }
	
		    return true;
		}
		else
		    return false;
    },

    cleanWindowsList: function()
    {
		for (let i = this.windowsList.length -1 ; i>=0 ; i--)
		    this.removeWindowInList(this.windowsList[i]);
    }
}